import os
import time
import gitlab
import logging


class ExportProjects():
    def __init__(self, gitlab_private_key, gitlab_private_url):
        self.gl = gitlab.Gitlab(gitlab_private_url, gitlab_private_key)

    def get_all_projects(self, main_group_id):
        group = self.gl.groups.get(main_group_id)
        projects = group.projects.list(include_subgroups=True, all=True)
        project_ids = []
        for project in projects:
            project_ids.append((project.path_with_namespace, project.id))
        return project_ids

    def export_all_projects(self, project_ids, export_timeout_project, export_path_project):
        for i, (project_name, project_id) in enumerate(project_ids):
            print(
                f"Starting to export project {i+1}/{len(project_ids)} - {project_name}")

            # Create export
            p = self.gl.projects.get(project_id)
            export = p.exports.create()

            # Wait for finished status
            export.refresh()
            t1 = time.time()
            while export.export_status != 'finished':
                if round(time.time()-t1) >= export_timeout_project:
                    break
                print(
                    f"Project {i+1}/{len(project_ids)} exporting since {round(time.time()-t1)} seconds...")
                time.sleep(1)
                export.refresh()

            # Download export
            filename = export_path_project + \
                str(project_name) + '-export.tar.gz'
            os.makedirs(os.path.dirname(filename), exist_ok=True)
            try:
                with open(filename, 'wb') as f:
                    export.download(streamed=True, action=f.write)
            except:
                logging.error(
                    f"Error exporting project {i+1}/{len(project_ids)} - {project_name}")



import os
import time
import gitlab
import logging


class ExportGroups():
    def __init__(self, gitlab_private_key, gitlab_private_url):
        self.gl = gitlab.Gitlab(gitlab_private_url, gitlab_private_key)

    def get_all_groups(self, main_group_id):
        main_group = self.gl.groups.get(main_group_id)
        main_group_name = main_group.full_path
        group_ids = [(main_group_name, main_group_id)]
        for (_, group_id) in group_ids:
            group = self.gl.groups.get(group_id)
            subgroups = group.subgroups.list()
            for sub in subgroups:
                group_ids.append((sub.full_path, sub.id))
        return group_ids

    def export_all_groups(self, group_ids, export_time_group, export_path_group):
        for i, (group_name, group_id) in enumerate(group_ids):
            print(
                f"Starting to export group {i+1}/{len(group_ids)} - {group_name}")

            # Create export
            g = self.gl.groups.get(group_id)
            export = g.exports.create()

            # Wait for finished status
            time.sleep(export_time_group)

            # Download export
            filename = export_path_group + \
                str(group_name) + '-export.tar.gz'
            os.makedirs(os.path.dirname(filename), exist_ok=True)
            try:
                with open(filename, 'wb') as f:
                    export.download(streamed=True, action=f.write)
            except:
                logging.error(
                    f"Error exporting group {i+1}/{len(group_ids)} - {group_name}")



from setuptools import setup, find_packages

setup(
    name='gitlab-group-project-exporter',
    version='1.0.4',
    license='MIT',
    description='Export gitlab groups and projects for remote backups',
    author='Muhammad Asad',
    author_email='muhammad.asad@gmail.com',
    url='https://github.com/masad8',
    packages=find_packages(),
    scripts=["bin/main.py"],
    keywords=['Gitlab', 'Exporter', 'Backup'],
    install_requires=[
        'python-gitlab >= 3.1.1',
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
)



import os
from datetime import datetime
from gitlabexporter.groupexport import ExportGroups
from gitlabexporter.projectexport import ExportProjects


if __name__ == "__main__":
    MAIN_GROUP_ID = 1234567
    GITLAB_PRIVATE_KEY = 'ABC'
    GITLAB_URL = 'https://gitlab.com'
    EXPORT_TIME_GROUP = 60  # seconds
    EXPORT_TIMEOUT_PROJECT = 3600  # seconds
    EXPORT_PATH_GROUP = os.environ['HOME'] + "/gitlab-backups/" + \
        datetime.today().strftime('%Y%m%d') + "/groups/"
    EXPORT_PATH_PROJECT = os.environ['HOME'] + "/gitlab-backups/" + \
        datetime.today().strftime('%Y%m%d') + "/projects/"

    p = ExportProjects(GITLAB_PRIVATE_KEY, GITLAB_URL)
    project_ids = p.get_all_projects(MAIN_GROUP_ID)
    p.export_all_projects(
        project_ids, EXPORT_TIMEOUT_PROJECT, EXPORT_PATH_PROJECT)

    g = ExportGroups(GITLAB_PRIVATE_KEY, GITLAB_URL)
    group_ids = g.get_all_groups(MAIN_GROUP_ID)
    g.export_all_groups(group_ids, EXPORT_TIME_GROUP, EXPORT_PATH_GROUP)
